source("stepSizeCalculator.R")

if (!is.loaded("logisticL2E")) {
  dyn.load("../Cfunctions/logisticL2E.so")  
} else {
	dyn.unload("logisticL2E.so")
	dyn.load("logisticL2E.so")	
}

call_logisticL2E = function(X, Y, alpha, lambda, beta0, beta, w = 1, niter=500, tol=1e-9) {
	if (!is.matrix(X)) stop("X must be a matrix.")

	n = dim(X)[1]
	p = dim(X)[2]

	K = stepSizeCalculator(w)
		
	if (length(Y) != n | length(beta) !=p) stop("length(Y) == nrow(X) & length(beta) == ncol(X)")
		
	print(paste("alpha = ",alpha, ", lambda = ",signif(lambda, digits=6)," | w = ", w, ",K = ",signif(K,digits=3)))
	print("==============================================================")
	output = .C("logisticL2E", as.double(X), as.integer(Y), as.double(beta0), as.double(beta),
		as.integer(n), as.integer(p), as.double(K), as.double(w), as.double(lambda), as.double(alpha),
		as.integer(niter),as.double(tol),beta_final=double(p+1))$beta_final
		
	return(output)
}

rLogistic = function(X, Y, alpha, lambda, beta0, beta, w = 1, niter=500, tol=1e-6, maxVar = 100) {

	modelID = c()
	for (i in 1:length(lambda)) {
		modelID[i] = paste("s", i-1,sep="")
	}
	model = list()
	model$a0 = numeric(length(lambda))
	names(model$a0) = modelID
	model$beta = matrix(NA, ncol(X), length(lambda))
	rownames(model$beta) = colnames(X)
	colnames(model$beta) = modelID
	model$df = rep(NA,length(lambda))
	model$lambda = lambda

	nIter = 0
	for (i in 1:length(lambda)) {
		nIter = nIter + 1
		output = call_logisticL2E(X, Y, alpha, lambda[i], beta0, beta, w=w, niter=niter, tol=tol)
		beta0 = output[1]
		beta = matrix(output[-1], ncol=1)
		model$a0[i] = beta0
		model$beta[,i] = beta
		varSel = which(abs(beta) > 0)
		model$df[i] = length(varSel)
		if (length(varSel) > maxVar)
			break
	}

	model$a0 = model$a0[1:nIter]
	model$beta = model$beta[,1:nIter]
	model$df = model$df[1:nIter]
	model$lambda = model$lambda[1:nIter]
	model$dim = dim(model$beta)
	return(model)
}
