# Script to create Figure 3.
library(ggplot2)
library(xtable)
files = dir(pattern=".rda")

for (i in 1:length(files)) {
	load(files[i])
	shift = unlist(strsplit(unlist(strsplit(files[i],"shift="))[2],".rda"))[1]
	l2e.wopt = cbind(apply(CD.wopt,1,mean),apply(CD.wopt,1,sd))
	l2e.w1 = cbind(apply(CD,1,mean),apply(CD,1,sd))
	mle = cbind(apply(MLE,1,mean),apply(MLE,1,sd))
	print(paste("*********** shift=",shift," ***********",sep=""))
	print(xtable(cbind(mle,l2e.w1),digits=3))
}

normData = data.frame()
for (i in 1:length(files)) {
	load(files[i])
	shift = unlist(strsplit(unlist(strsplit(files[i],"shift="))[2],".rda"))[1]
	CD.norm = apply(as.matrix(CD[-1,,drop=F]),2,FUN=function(x){sqrt(sum(x^2))})
	temp.df = data.frame(norm = CD.norm)
	temp.df$shift = shift
	temp.df$method = "l2ew=1"
	normData = rbind(normData, temp.df)	
	MLE.norm = apply(as.matrix(MLE[-1,,drop=F]),2,FUN=function(x){sqrt(sum(x^2))})
	temp.df = data.frame(norm = MLE.norm)
	temp.df$shift = shift
	temp.df$method = "mle"
	normData = rbind(normData, temp.df)	
}

normData$method = factor(normData$method, levels=c("mle","l2ew=1"), ordered=TRUE)
limits = c("mle","l2ew=1")
labels = c(expression(paste("MLE")), expression(paste(L[2],"E")))

normData$shift = ordered(normData$shift, levels=c(-0.25,1.5,3,6,12,24))
q = ggplot(data=normData, aes(x=shift, y=norm))
q = q + geom_boxplot(aes(fill=method)) + theme_bw()
q = q + scale_fill_manual(name="Method", values=c("#1F78B4","#B2DF8A"), breaks=limits, labels=labels)
q = q + xlab("Outlier Position") + ylab(expression(paste("2-norm of ", beta)))
q = q + ylim(c(0,max(normData$norm)))
# Decided not to add the norm of the true beta because of Jensen's inequality.
#q + geom_hline(yintercept=sqrt(sum(beta.true[-1]^2))) 
q + opts(axis.title.x=theme_text(vjust=-0.25))
ggsave("normVsShift.pdf", width=8, height=3)
