# Script to generate Figures 1 and 2
# An example of breakdown causing the regularization path to fail.
library(glmnet)
library(ggplot2)
pwd = getwd()
setwd("../../Rfunctions")
source("rLogistic.R")
source("calculateLambdaSequence.R")
setwd(pwd)

set.seed(1)
n = 200
mu = 1.2
centroids = matrix(c(rep(mu,n/2),rep(-mu,n/2)),ncol=1)
X = matrix(rnorm(n),ncol=1) + centroids
beta = 1.25

Y = ifelse(runif(n) <= plogis(beta*X),1,0)
fit0 = glm(Y~.,family=binomial(logit),data=data.frame(X=X,Y=Y))
x.new = seq(-11,11,0.01)
df.mle = data.frame(x = x.new, fit = plogis(predict(fit0,newdata=data.frame(X = x.new))))
df.mle$nc = 0
df.true = data.frame(x=x.new, y = plogis(beta*x.new))
df.true$nc = 0
df.points = data.frame(X=X,Y=Y)
df.points$nc = 0

Z = apply(X,2,FUN=function(x){x-mean(x)})
Zm = apply(X,2,mean)
system.time({fit0.l2e = rLogistic(Z,Y,alpha=1,lambda=0,beta0=1,beta=1,maxVar=10, w=1)})
df.l2e = data.frame(x = x.new, fit = plogis(fit0.l2e$a0 + fit0.l2e$beta*x.new- Zm*fit0.l2e$beta))
df.l2e$nc = 0

# Add dummy variables
p = 999
noise = matrix(rnorm(n*p),n,p)
Xn = cbind(X, noise)

glmnet.fit = glmnet(Xn,Y,family="binomial", alpha = 1)
L1norm = apply(as.matrix(glmnet.fit$beta),2,FUN=function(x){sum(abs(x))})
lambda = glmnet.fit$lambda
active_set = which(apply(glmnet.fit$beta,1,function(x){sum(abs(x))}) > 0)
data=data.frame(beta = t(as.matrix(glmnet.fit$beta[active_set,])), L1norm=L1norm, lambda=lambda)
melted = melt(data, id.vars=c("L1norm","lambda"))
ix1 = which(melted$variable == "beta.V1")
melted$truth = 0
melted$truth[ix1] = 1
melted$truth = factor(melted$truth, levels=c(0,1))
melted$nc = 0
outlier.df = melted

### L2E
Xn = apply(Xn,2,FUN=function(x){x-mean(x)})
lambda = calculateLambdaSequence(Y,Xn,alpha=1,w=1,epsilon=0.015,nLambda=100)
beta = matrix(1,p+1,1)
beta0 = log(mean(Y)/(1-mean(Y)))
system.time({rLogistic.fit = rLogistic(Xn,Y,alpha=1,lambda,beta0,beta,maxVar=1000, w=1)})
L1norm.l2e = apply(rLogistic.fit$beta,2,FUN=function(x){sum(abs(x))})
active_set = which(apply(rLogistic.fit$beta,1,function(x){sum(abs(x))}) > 0)
data.l2e = data.frame(beta = t(rLogistic.fit$beta[active_set,]), L1norm=L1norm.l2e, lambda=lambda)
melted.l2e = melt(data.l2e, id.vars=c("L1norm","lambda"))
ix1 = which(melted.l2e$variable == "beta.1")
melted.l2e$truth = 0
melted.l2e$truth[ix1] = 1
melted.l2e$truth = factor(melted.l2e$truth, levels=c(0,1))
melted.l2e$nc = 0
outlier.l2e = melted.l2e

# Add outlier
set.seed(1)
n = 200
mu = 1.2
centroids = matrix(c(rep(mu,n/2),rep(-mu,n/2)),ncol=1)
X = matrix(rnorm(n),ncol=1) + centroids
beta = 1.25

Y = ifelse(runif(n) <= plogis(beta*X),1,0)

nc = 5
Y[(n+1):(n+nc)] = 0
X[(n+1):(n+nc)] = jitter(rep(10,nc))
X = as.matrix(X)

fit5 = glm(Y~.,family=binomial(logit),data=data.frame(X=X,Y=Y))
temp = data.frame(x = x.new, fit = plogis(predict(fit5, newdata=data.frame(X = x.new))))
temp$nc = nc
df.mle = rbind(df.mle, temp)
temp = data.frame(x=x.new, y = plogis(beta*x.new))
temp$nc = nc
df.true = rbind(df.true, temp)
temp = data.frame(X=X,Y=Y)
temp$nc = nc
df.points = rbind(df.points, temp)

Z = apply(X,2,FUN=function(x){x-mean(x)})
Zm = apply(X,2,mean)
system.time({fit5.l2e = rLogistic(Z,Y,alpha=1,lambda=0,beta0=1,beta=1,maxVar=10, w=1)})
temp = data.frame(x = x.new, fit = plogis(fit5.l2e$a0 + fit5.l2e$beta*x.new - Zm*fit5.l2e$beta))
temp$nc = nc
df.l2e = rbind(df.l2e, temp)

# Add dummy variables
p = 999
noise.out = matrix(rnorm(nc*p),nc,p)
noise.out = rbind(noise,noise.out)
Xn = cbind(X, noise.out)

glmnet.fit.out = glmnet(Xn,Y,family="binomial", alpha = 1)

L1norm = apply(as.matrix(glmnet.fit.out$beta),2,FUN=function(x){sum(abs(x))})
lambda = glmnet.fit.out$lambda
active_set = which(apply(glmnet.fit.out$beta,1,function(x){sum(abs(x))}) > 0)
data=data.frame(beta = t(as.matrix(glmnet.fit.out$beta[active_set,])), L1norm=L1norm, lambda=lambda)
melted = melt(data, id.vars=c("L1norm","lambda"))
ix1 = which(melted$variable == "beta.V1")
melted$truth = 0
melted$truth[ix1] = 1
melted$truth = factor(melted$truth, levels=c(0,1))
melted$nc = nc

outlier.df = rbind(outlier.df, melted)

## L2E
Xn = apply(Xn,2,FUN=function(x){x-mean(x)})
lambda = calculateLambdaSequence(Y,Xn,alpha=1,w=1,epsilon=0.015,nLambda=100)
beta = matrix(0,p+1,1)
beta0 = log(mean(Y)/(1-mean(Y)))
system.time({rLogistic.fit = rLogistic(Xn,Y,alpha=1,lambda,beta0,beta,maxVar=1000, w=1)})
L1norm.l2e = apply(rLogistic.fit$beta,2,FUN=function(x){sum(abs(x))})
active_set = which(apply(rLogistic.fit$beta,1,function(x){sum(abs(x))}) > 0)
data.l2e = data.frame(beta = t(rLogistic.fit$beta[active_set,]), L1norm=L1norm.l2e, lambda=lambda)
melted.l2e = melt(data.l2e, id.vars=c("L1norm","lambda"))
ix1 = which(melted.l2e$variable == "beta.1")
melted.l2e$truth = 0
melted.l2e$truth[ix1] = 1
melted.l2e$truth = factor(melted.l2e$truth, levels=c(0,1))
melted.l2e$nc = nc
outlier.l2e = rbind(outlier.l2e, melted.l2e)

# Add outlier
set.seed(1)
n = 200
mu = 1.2
centroids = matrix(c(rep(mu,n/2),rep(-mu,n/2)),ncol=1)
X = matrix(rnorm(n),ncol=1) + centroids
beta = 1.25

Y = ifelse(runif(n) <= plogis(beta*X),1,0)

nc = 10
Y[(n+1):(n+nc)] = 0
X[(n+1):(n+nc)] = jitter(rep(10,nc))
X = as.matrix(X)

fit10 = glm(Y~.,family=binomial(logit),data=data.frame(X=X,Y=Y))
temp = data.frame(x = x.new, fit = plogis(predict(fit10, newdata=data.frame(X = x.new))))
temp$nc = nc
df.mle = rbind(df.mle, temp)
temp = data.frame(x=x.new, y = plogis(beta*x.new))
temp$nc = nc
df.true = rbind(df.true, temp)
temp = data.frame(X=X,Y=Y)
temp$nc = nc
df.points = rbind(df.points, temp)

Z = apply(X,2,FUN=function(x){x-mean(x)})
Zm = apply(X,2,mean)
system.time({fit10.l2e = rLogistic(Z,Y,alpha=1,lambda=0,beta0=1,beta=1,maxVar=10, w=1)})
temp = data.frame(x = x.new, fit = plogis(fit10.l2e$a0 + fit10.l2e$beta*x.new - Zm*fit5.l2e$beta))
temp$nc = nc
df.l2e = rbind(df.l2e, temp)

red_colour = rgb(215/255, 25/255, 28/255)

q = ggplot(data = df.mle)
q = q + geom_line(aes(x = x.new, y = fit), colour=red_colour, size=1.5, alpha=0.75)
q = q + geom_line(data = df.true, aes(x=x.new,y=y), linetype=2)
q = q + geom_point(data = df.points,aes(x=X, y=Y))
q = q + ylab(expression(paste("Pr(Y=1|",X[1],"=",x[1],")"))) + xlab(expression(x[1])) + facet_grid(.~nc)
q + theme_bw() + opts(axis.title.x=theme_text(vjust=-0.25))
ggsave(filename="implosion_univariate_fits.pdf",width=8, height=3)

q = ggplot(data = df.l2e)
q = q + geom_line(aes(x = x.new, y = fit), colour=red_colour, size=1.5, alpha=0.75)
q = q + geom_line(data = df.true, aes(x=x.new,y=y), linetype=2)
q = q + geom_point(data = df.points,aes(x=X, y=Y))
q = q + ylab(expression(paste("Pr(Y=1|",X[1],"=",x[1],")"))) + xlab(expression(x[1])) + facet_grid(.~nc) 
q + theme_bw() + opts(axis.title.x=theme_text(vjust=-0.25))
ggsave(filename="implosion_univariate_fits_l2e.pdf",width=8, height=3)

# Add dummy variables
p = 999
noise.out = matrix(rnorm(nc*p),nc,p)
noise.out = rbind(noise,noise.out)
Xn = cbind(X, noise.out)

glmnet.fit.out = glmnet(Xn,Y,family="binomial", alpha = 1)

L1norm = apply(as.matrix(glmnet.fit.out$beta),2,FUN=function(x){sum(abs(x))})
lambda = glmnet.fit.out$lambda
active_set = which(apply(glmnet.fit.out$beta,1,function(x){sum(abs(x))}) > 0)
data=data.frame(beta = t(as.matrix(glmnet.fit.out$beta[active_set,])), L1norm=L1norm, lambda=lambda)
melted = melt(data, id.vars=c("L1norm","lambda"))
ix1 = which(melted$variable == "beta.V1")
melted$truth = 0
melted$truth[ix1] = 1
melted$truth = factor(melted$truth, levels=c(0,1))
melted$nc = nc

outlier.df = rbind(outlier.df, melted)

## L2E
Xn = apply(Xn,2,FUN=function(x){x-mean(x)})
lambda = calculateLambdaSequence(Y,Xn,alpha=1,w=1,epsilon=0.015,nLambda=100)
beta = matrix(0,p+1,1)
beta0 = log(mean(Y)/(1-mean(Y)))
system.time({rLogistic.fit = rLogistic(Xn,Y,alpha=1,lambda,beta0,beta,maxVar=1000, w=1)})
L1norm.l2e = apply(rLogistic.fit$beta,2,FUN=function(x){sum(abs(x))})
active_set = which(apply(rLogistic.fit$beta,1,function(x){sum(abs(x))}) > 0)
data.l2e = data.frame(beta = t(rLogistic.fit$beta[active_set,]), L1norm=L1norm.l2e, lambda=lambda)
melted.l2e = melt(data.l2e, id.vars=c("L1norm","lambda"))
ix1 = which(melted.l2e$variable == "beta.1")
melted.l2e$truth = 0
melted.l2e$truth[ix1] = 1
melted.l2e$truth = factor(melted.l2e$truth, levels=c(0,1))
melted.l2e$nc = nc
outlier.l2e = rbind(outlier.l2e, melted.l2e)

q = ggplot(data=outlier.df, aes(x=L1norm, y=value, group=variable)) + geom_line(aes(colour=truth),alpha=0.4,size=0.5)
q = q + scale_colour_manual(values=c("black",red_colour))
ix_1 = which(outlier.df$truth==1)
q = q + geom_line(data=outlier.df[ix_1,],aes(x=L1norm, y=value), colour=red_colour, size=1)
q = q + ylab("Coefficients") + xlab(expression(paste("J(",beta,")")))
q = q + xlim(c(0,10)) + ylim(c(-0.5,1.5)) + facet_grid(.~nc, scales='free_x')
q = q + theme_bw() + opts(legend.position="none", axis.title.x=theme_text(vjust=-0.25))
q

ggsave(filename="implosion_rpath.pdf", width=8, height=3)

q = ggplot(data=outlier.l2e, aes(x=L1norm, y=value, group=variable)) + geom_line(aes(colour=truth),alpha=0.4,size=0.5)
q = q + scale_colour_manual(values=c("black",red_colour))
ix_1 = which(outlier.l2e$variable=="beta.1")
q = q + geom_line(data=outlier.l2e[ix_1,],aes(x=L1norm, y=value), colour=red_colour, size=1)
q = q + ylab("Coefficients") + xlab(expression(paste("J(",beta,")")))
q = q + xlim(c(0,10)) + ylim(c(-0.5,1.5)) + facet_grid(.~nc, scales='free_x')
q = q + theme_bw() + opts(legend.position="none", axis.title.x=theme_text(vjust=-0.25))
q
ggsave(filename="implosion_rpath_l2e.pdf", width=8, height=3)
